@@ -1,15 +1,14 @@
 ## Produce the correlation due to similarity of responses to X
-get.enviro.cor <- function(object, est = "median", prob = 0.95) 
-	{
+get.enviro.cor <- function(object, est = "median", prob = 0.95) {
 
      if(is.null(object$jags.model)) 
-          stop("MCMC samples not found")
+          stop("MCMC samples not found.")
      fit.mcmc <- get.mcmcsamples(object)
      y <- object$y
      X <- object$X
      
      if(length(grep("X.coefs", colnames(fit.mcmc))) == 0) 
-          stop("Cannot find MCMC sample corresponding to coefficients for X")
+          stop("Cannot find MCMC sample corresponding to coefficients for X.")
 
      n <- nrow(y); p <- ncol(y)
      enviro_cor_mat <- enviro_cor_mat_cilower <- enviro_cor_mat_ciupper <- enviro_cov_mat <- matrix(0,p,p)
@@ -21,34 +20,30 @@ get.enviro.cor <- function(object, est = "median", prob = 0.95)
      all_enviro_cov_mat <- all_enviro_cor_mat <- array(0, dim = c(nrow(fit.mcmc),p,p))
 
     
-     for(k0 in 1:nrow(fit.mcmc)) 
-		{
+     for(k0 in 1:nrow(fit.mcmc)) {
 		cw_X_coefs <- matrix(fit.mcmc[k0,grep("X.coefs", colnames(fit.mcmc))], nrow=p)
 		enviro.linpreds <- tcrossprod(X,as.matrix(cw_X_coefs))
 		all_enviro_cov_mat[k0,,] <- cov(enviro.linpreds)
 		all_enviro_cor_mat[k0,,] <- cor(enviro.linpreds) 
           }
 
-     for(j in 1:p) { for(j2 in 1:p) 
-		{ ## Average/Median over the MCMC samples
-		if(est == "median") 
-				{ 
-				enviro_cov_mat[j,j2] <- median(all_enviro_cov_mat[,j,j2])
-				enviro_cor_mat[j,j2] <- median(all_enviro_cor_mat[,j,j2]) 
-				}
-		if(est == "mean") 
-				{
-				enviro_cov_mat[j,j2] <- mean(all_enviro_cov_mat[,j,j2])
-				enviro_cor_mat[j,j2] <- mean(all_enviro_cor_mat[,j,j2]) 
-				} 
+     for(j in 1:p) { for(j2 in 1:p) { ## Average/Median over the MCMC samples
+		if(est == "median") { 
+               enviro_cov_mat[j,j2] <- median(all_enviro_cov_mat[,j,j2])
+               enviro_cor_mat[j,j2] <- median(all_enviro_cor_mat[,j,j2]) 
+               }
+		if(est == "mean") {
+               enviro_cov_mat[j,j2] <- mean(all_enviro_cov_mat[,j,j2])
+               enviro_cor_mat[j,j2] <- mean(all_enviro_cor_mat[,j,j2]) 
+               } 
             
-            sig_enviro_cor_mat[j,j2] <- enviro_cor_mat[j,j2]
-            get.hpd.cors <- HPDinterval(as.mcmc(all_enviro_cor_mat[,j,j2]), prob = prob)
-            enviro_cor_mat_cilower[j,j2] <- get.hpd.cors[1] 
-            enviro_cor_mat_ciupper[j,j2] <- get.hpd.cors[2]
-            if(0 > get.hpd.cors[1] & 0 < get.hpd.cors[2]) 
-				sig_enviro_cor_mat[j,j2] <- 0
-		} }
+          sig_enviro_cor_mat[j,j2] <- enviro_cor_mat[j,j2]
+          get.hpd.cors <- HPDinterval(as.mcmc(all_enviro_cor_mat[,j,j2]), prob = prob)
+          enviro_cor_mat_cilower[j,j2] <- get.hpd.cors[1] 
+          enviro_cor_mat_ciupper[j,j2] <- get.hpd.cors[2]
+          if(0 > get.hpd.cors[1] & 0 < get.hpd.cors[2]) 
+               sig_enviro_cor_mat[j,j2] <- 0
+          } }
             
      #return(list(residual.correlations = enviro_cor_mat))
      #corrplot(enviro_cor_mat, title = "Environmental correlations", type = "lower")
@@ -57,10 +52,9 @@ get.enviro.cor <- function(object, est = "median", prob = 0.95)
 
 	
 ## Produce the residual correlation based on latent variables
-get.residual.cor <- function(object, est = "median", prob = 0.95) 
-	{
+get.residual.cor <- function(object, est = "median", prob = 0.95) {
      if(is.null(object$jags.model)) 
-          stop("MCMC samples not found")
+          stop("MCMC samples not found.")
 
      fit.mcmc <- get.mcmcsamples(object)
      y <- object$y
@@ -68,7 +62,7 @@ get.residual.cor <- function(object, est = "median", prob = 0.95)
      num.lv <- object$num.lv
 
      if(length(grep("lvs", colnames(fit.mcmc))) == 0) 
-          stop("Cannot find MCMC samples corresponding to latent variables")
+          stop("Cannot find MCMC samples corresponding to latent variables.")
 
      n <- nrow(y)
      p <- ncol(y)
@@ -82,8 +76,7 @@ get.residual.cor <- function(object, est = "median", prob = 0.95)
      all_rescor_mat <- all.rescov_mat <- all_respres_mat <- array(0, dim = c(nrow(fit.mcmc),p,p))
      all_trace_rescor <- numeric(nrow(fit.mcmc))
 
-     for(k0 in 1:nrow(fit.mcmc)) 
-          {
+     for(k0 in 1:nrow(fit.mcmc)) {
 		lv.coefs <- matrix(fit.mcmc[k0,grep("lv.coefs", colnames(fit.mcmc))],nrow=p)
 # 		if(all(object$family == "binomial") & all(object$trial.size == 1)) 
 # 			lv.coefs[,2:(num.lv+1)] <- lv.coefs[,2:(num.lv+1)]/matrix(sqrt(1-rowSums(lv.coefs[,2:(num.lv+1)]^2)),nrow=p,ncol=num.lv,byrow=FALSE) ## If data is Bernoulli, then scale the coefficients to acocunt for constraints (see Knott and Bartholomew, Chapter 4)
@@ -103,36 +96,33 @@ get.residual.cor <- function(object, est = "median", prob = 0.95)
 		all_respres_mat[k0,,] <- cor2pcor(lambdalambdaT)
 		}
             
-     for(j in 1:p) { for(j2 in 1:p) 
-		{ ## Average/Median over the MCMC samples
-            if(est == "median") 
-				{ 
-				rescov_mat[j,j2] <- median(all.rescov_mat[,j,j2]) 
-				rescor_mat[j,j2] <- median(all_rescor_mat[,j,j2]); 
-				respres_mat[j,j2] <- median(all_respres_mat[,j,j2]); 
-				}
+     for(j in 1:p) { for(j2 in 1:p) { ## Average/Median over the MCMC samples
+          if(est == "median") { 
+          rescov_mat[j,j2] <- median(all.rescov_mat[,j,j2]) 
+          rescor_mat[j,j2] <- median(all_rescor_mat[,j,j2]); 
+          respres_mat[j,j2] <- median(all_respres_mat[,j,j2]); 
+               }
             
-            if(est == "mean") 
-				{ 
-				rescov_mat[j,j2] <- mean(all.rescov_mat[,j,j2]) 
-				rescor_mat[j,j2] <- mean(all_rescor_mat[,j,j2]); 
-				respres_mat[j,j2] <- mean(all_respres_mat[,j,j2]); 
-				}
-            
-            sig_rescor_mat[j,j2] <- rescor_mat[j,j2]
-            get.hpd.cors <- HPDinterval(as.mcmc(all_rescor_mat[,j,j2]), prob = prob)
-			rescor_mat_cilower[j,j2] <- get.hpd.cors[1]
-            rescor_mat_ciupper[j,j2] <- get.hpd.cors[2]
-            if(0 > get.hpd.cors[1] & 0 < get.hpd.cors[2]) 
-				sig_rescor_mat[j,j2] <- 0
+          if(est == "mean") { 
+          rescov_mat[j,j2] <- mean(all.rescov_mat[,j,j2]) 
+          rescor_mat[j,j2] <- mean(all_rescor_mat[,j,j2]); 
+          respres_mat[j,j2] <- mean(all_respres_mat[,j,j2]); 
+               }
+               
+          sig_rescor_mat[j,j2] <- rescor_mat[j,j2]
+          get.hpd.cors <- HPDinterval(as.mcmc(all_rescor_mat[,j,j2]), prob = prob)
+          rescor_mat_cilower[j,j2] <- get.hpd.cors[1]
+          rescor_mat_ciupper[j,j2] <- get.hpd.cors[2]
+          if(0 > get.hpd.cors[1] & 0 < get.hpd.cors[2]) 
+               sig_rescor_mat[j,j2] <- 0
 
-            sig_respres_mat[j,j2] <- respres_mat[j,j2]
-            get.hpd.cors <- HPDinterval(as.mcmc(all_respres_mat[,j,j2]), prob = prob)
-			respres_mat_cilower[j,j2] <- get.hpd.cors[1]
-            respres_mat_ciupper[j,j2] <- get.hpd.cors[2]
-            if(0 > get.hpd.cors[1] & 0 < get.hpd.cors[2]) 
-				sig_respres_mat[j,j2] <- 0
-		} }
+          sig_respres_mat[j,j2] <- respres_mat[j,j2]
+          get.hpd.cors <- HPDinterval(as.mcmc(all_respres_mat[,j,j2]), prob = prob)
+          respres_mat_cilower[j,j2] <- get.hpd.cors[1]
+          respres_mat_ciupper[j,j2] <- get.hpd.cors[2]
+          if(0 > get.hpd.cors[1] & 0 < get.hpd.cors[2]) 
+               sig_respres_mat[j,j2] <- 0
+          } }
 
      if(est == "median") 
           final_trace <- median(all_trace_rescor)
